@@ -2,8 +2,9 @@ module RNG where
 
 import Data.Word
 import Data.List (foldl')
+import Data.Byteable
 import qualified Data.ByteString as B
-import Crypto.Random.API
+import Crypto.Random
 import Control.Arrow (first)
 
 {- this is a just test rng. this is absolutely not a serious RNG. DO NOT use elsewhere -}
@@ -24,9 +25,20 @@ getBytes n g =
      in (b:l, g'')
 
 instance CPRG Rng where
-    cprgGenBytes len g    = first B.pack $ getBytes len g
-    cprgSupplyEntropy e g = reseed e g
-    cprgNeedReseed _      = NeverReseed
+    cprgCreate pool = let ent = grabEntropy 4 pool in generate (B.unpack $ toBytes ent)
+    cprgSetReseedThreshold _ g = g
+    cprgGenerate len g = first B.pack $ getBytes len g
+    cprgGenerateWithEntropy = cprgGenerate
+    cprgFork g = let (bs, g') = getBytes 4 g
+                    in case bs of
+                        [a,b,c,d] -> let g2 = Rng (fromIntegral a * 256 + fromIntegral b, fromIntegral c * 256 + fromIntegral d)
+                                      in (g2, g')
+                        _         -> error "getBytes assertion"
+
+
+generate :: [Word8] -> Rng
+generate [a,b,c,d] = Rng (fromIntegral a * 256 + fromIntegral b, fromIntegral c * 256 + fromIntegral d)
+generate _ = error "generate assertion: need 4 bytes"
 
 reseed :: B.ByteString -> Rng -> Rng
 reseed bs (Rng (a,b)) = Rng (fromIntegral a', b')
@@ -35,4 +47,4 @@ reseed bs (Rng (a,b)) = Rng (fromIntegral a', b')
               l  = B.unpack bs
 
 rng :: Rng
-rng = Rng (1,2) 
+rng = Rng (2,2)
